﻿// cc-by 3.0 Austria, (c) 2011 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package at.ssw.jku.view;

import at.ssw.jku.model.Field;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.binding.IntegerExpression;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.shape.Line;

public final class FieldNode extends Parent {
  private static final int BLOCK = 10;
  private final int width;
  private final int height;
  private final int MIN_X = 0;
  private final int MIN_Y = 0;
  private final int MAX_X = Field.COLUMNS * BLOCK;
  private final int MAX_Y = Field.ROWS * BLOCK;
  private final CoordinateConverter converter = new FieldCoordinateConverter(BLOCK);
  private final WormNode worm;
  private final GoodyNode goody;

  public FieldNode(Field field) {
    width = Field.COLUMNS * BLOCK + 1;
    height = Field.ROWS * BLOCK + 1;
    worm = new WormNode(field.getWorm(), converter);
    goody = new GoodyNode(field.getGoody(), converter);
    initChildren();
  }

  private void initChildren() {
    getChildren().add(worm);
    getChildren().addAll(createRows());
    getChildren().addAll(createColumns());
    getChildren().add(goody);
  }

  private Node[] createRows() {
    final Node[] rows = new Node[Field.ROWS + 1];

    for (int row = 0; row <= Field.ROWS; ++row) {
      final double y = converter.modelToScreenY(row);
      rows[row] = new Line(MIN_X, y, MAX_X, y);
    }

    return rows;
  }

  private Node[] createColumns() {
    final Node[] columns = new Node[Field.COLUMNS + 1];

    for (int col = 0; col <= Field.COLUMNS; ++col) {
      final double x = converter.modelToScreenX(col);
      columns[col] = new Line(x, MIN_Y, x, MAX_Y);
    }

    return columns;
  }

  public int getHeight() {
    return height;
  }

  public int getWidth() {
    return width;
  }

  private static final class FieldCoordinateConverter implements CoordinateConverter {
    private final int block;

    public FieldCoordinateConverter(int block) {
      this.block = block;
    }

    @Override
    public double modelToScreenX(int modelX) {
      return modelX * block;
    }

    @Override
    public double modelToScreenY(int modelY) {
      return modelY * block;
    }

    @Override
    public double modelToScreenWidth(int width) {
      return width * block;
    }

    @Override
    public double modelToScreenHeight(int height) {
      return height * block;
    }

    @Override
    public DoubleBinding modelToScreenXBinding(final IntegerExpression modelX) {
      return new DoubleBinding() {
        {
          super.bind(modelX);
        }

        @Override
        protected double computeValue() {
          return modelToScreenX(modelX.getValue());
        }
      };
    }

    @Override
    public DoubleBinding modelToScreenYBinding(final IntegerExpression modelY) {
      return new DoubleBinding() {
        {
          super.bind(modelY);
        }

        @Override
        protected double computeValue() {
          return modelToScreenY(modelY.getValue());
        }
      };
    }

    @Override
    public DoubleBinding modelToScreenWidthBinding(final IntegerExpression modelWidth) {
      return new DoubleBinding() {
        {
          super.bind(modelWidth);
        }

        @Override
        protected double computeValue() {
          return modelToScreenWidth(modelWidth.getValue());
        }
      };
    }

    @Override
    public DoubleBinding modelToScreenHeightBinding(final IntegerExpression modelHeight) {
      return new DoubleBinding() {
        {
          super.bind(modelHeight);
        }

        @Override
        protected double computeValue() {
          return modelToScreenWidth(modelHeight.getValue());
        }
      };
    }
  }
}
